<?php

namespace Drupal\upsync;

use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Result collector.
 */
class ResultCollector implements ResultCollectorInterface {

  /**
   * Storage for messages.
   *
   * @var array
   */
  protected $messages;

  /**
   * Storage for errors.
   *
   * @var array
   */
  protected $errors;

  /**
   * Contructs new ResultCollector.
   */
  public function __construct() {
    $this->messages = [];
    $this->errors = [];
  }

  /**
   * {@inheritdoc}
   */
  public function addMessage(string|TranslatableMarkup $message, string $category = 'general') {
    $this->messages[$category][] = $message;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function addError(string|TranslatableMarkup $message, string $category = 'general') {
    $this->errors[$category][] = $message;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getMessages(?string $category = NULL) {
    return $category ? ($this->messages[$category] ?? []) : $this->flatten($this->messages);
  }

  /**
   * {@inheritdoc}
   */
  public function getErrors(?string $category = NULL) {
    return $category ? ($this->errors[$category] ?? []) : $this->flatten($this->errors);
  }

  /**
   * {@inheritdoc}
   */
  public function hasErrors() {
    return !empty($this->errors);
  }

  /**
   * {@inheritdoc}
   */
  public function clear() {
    $this->messages = [];
    $this->errors = [];
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function all() {
    return [
      'messages' => $this->getMessages(),
      'errors' => $this->getErrors(),
    ];
  }

  /**
   * Flattens the given array.
   */
  protected function flatten(array $nested) {
    return array_merge(...array_values($nested));
  }

}
